"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.jitPlugin = void 0;
function jitPlugin({ inlineStylesExtension, }) {
    let styleTransform;
    let watchMode = false;
    let viteServer;
    let cssPlugin;
    return {
        name: '@analogjs/vite-plugin-angular-jit',
        config(_config, { command }) {
            watchMode = command === 'serve';
        },
        buildStart({ plugins }) {
            if (Array.isArray(plugins)) {
                cssPlugin = plugins.find((plugin) => plugin.name === 'vite:css');
            }
            styleTransform = watchMode
                ? viteServer.pluginContainer.transform
                : cssPlugin.transform;
        },
        configureServer(server) {
            viteServer = server;
        },
        resolveId(id) {
            if (id.startsWith('virtual:angular')) {
                return `\0${id}`;
            }
            return;
        },
        async load(id) {
            if (id.includes('virtual:angular:jit:style:inline;')) {
                const styleId = id.split('style:inline;')[1];
                const decodedStyles = Buffer.from(decodeURIComponent(styleId), 'base64').toString();
                let styles = '';
                try {
                    const compiled = await styleTransform(decodedStyles, `${styleId}.${inlineStylesExtension}?direct`);
                    styles = compiled?.code;
                }
                catch (e) {
                    console.error(`${e}`);
                }
                return `export default \`${styles}\``;
            }
            return;
        },
    };
}
exports.jitPlugin = jitPlugin;
//# sourceMappingURL=angular-jit-plugin.js.map