"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MarkedSetupService = void 0;
/**
 * Credit goes to Scully for original implementation
 * https://github.com/scullyio/scully/blob/main/libs/scully/src/lib/fileHanderPlugins/markdown.ts
 */
const marked_1 = require("marked");
const marked_gfm_heading_id_1 = require("marked-gfm-heading-id");
const marked_highlight_1 = require("marked-highlight");
require("prismjs");
require("prismjs/components/prism-bash");
require("prismjs/components/prism-css");
require("prismjs/components/prism-javascript");
require("prismjs/components/prism-json");
require("prismjs/components/prism-markup");
require("prismjs/components/prism-typescript");
require("prismjs/plugins/copy-to-clipboard/prism-copy-to-clipboard");
require("prismjs/plugins/toolbar/prism-toolbar");
class MarkedSetupService {
    marked;
    constructor() {
        const renderer = new marked_1.marked.Renderer();
        renderer.code = (code, lang) => {
            // Escape commonly used HTML characters
            // in Angular templates that cause template parse errors
            // such as @, {, and ,}.
            code = code.replace(/@/g, '&#64;');
            if (code.includes('>{<') || code.includes('>}<')) {
                code = code
                    .replace(/>\{</g, '>&#x2774;<')
                    .replace(/>\}</g, '>&#x2775;<');
            }
            // Let's do a language based detection like on GitHub
            // So we can still have non-interpreted mermaid code
            if (lang === 'mermaid') {
                return '<pre class="mermaid">' + code + '</pre>';
            }
            if (!lang) {
                return '<pre><code>' + code + '</code></pre>';
            }
            const classes = lang.startsWith('diff') && Prism.languages['diff']
                ? `language-${lang} diff-highlight`
                : `language-${lang.replace('diff-', '')}`;
            return `<pre class="${classes}"><code class="${classes}">${code}</code></pre>`;
        };
        marked_1.marked.use((0, marked_gfm_heading_id_1.gfmHeadingId)(), (0, marked_highlight_1.markedHighlight)({
            async: true,
            highlight: (code, lang) => {
                let diff = lang?.startsWith('diff-');
                lang = diff ? lang.replace('diff-', '') : lang || 'typescript';
                if (diff && !Prism.languages['diff']) {
                    diff = false;
                    console.warn(`Notice:
    ---------------------------------------------------------------------------------------
    The \`diff\` language and plugin are not available in the provided setup.
    To enable it, add the following imports your \`main.ts\`:
      import 'prismjs/components/prism-diff';
      import 'prismjs/plugins/diff-highlight/prism-diff-highlight';
    ---------------------------------------------------------------------------------------
            `);
                }
                if (!Prism.languages[lang]) {
                    if (lang !== 'mermaid') {
                        console.warn(`Notice:
    ---------------------------------------------------------------------------------------
    The requested language '${lang}' is not available in the provided setup.
    To enable it, add the following import your \`main.ts\`:
      import 'prismjs/components/prism-${lang}';
    ---------------------------------------------------------------------------------------
              `);
                    }
                    return code;
                }
                return Prism.highlight(code, diff ? Prism.languages['diff'] : Prism.languages[lang], lang);
            },
        }), {
            async: false,
            renderer,
            pedantic: false,
            gfm: true,
            breaks: false,
            sanitize: false,
            smartypants: false,
            xhtml: false,
            mangle: false,
        });
        this.marked = marked_1.marked;
    }
    getMarkedInstance() {
        return this.marked;
    }
}
exports.MarkedSetupService = MarkedSetupService;
//# sourceMappingURL=marked-setup.service.js.map