"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.augmentHostWithResources = void 0;
const paths_1 = require("@ngtools/webpack/src/ivy/paths");
const node_fs_1 = require("node:fs");
const ts = require("typescript");
const analog_1 = require("./authoring/analog");
const marked_setup_service_1 = require("./authoring/marked-setup.service");
const constants_1 = require("./authoring/constants");
function augmentHostWithResources(host, transform, options = {}) {
    const resourceHost = host;
    const baseGetSourceFile = resourceHost.getSourceFile.bind(resourceHost);
    if (options.supportAnalogFormat) {
        resourceHost.getSourceFile = (fileName, languageVersionOrOptions, onError, ...parameters) => {
            if (fileName.includes('.analog')) {
                const contents = (0, node_fs_1.readFileSync)(fileName.replace('.analog.ts', '.analog'), 'utf-8');
                const source = (0, analog_1.compileAnalogFile)(fileName, contents, options.isProd);
                return ts.createSourceFile(fileName, source, languageVersionOrOptions, onError, ...parameters);
            }
            return baseGetSourceFile.call(resourceHost, fileName, languageVersionOrOptions, onError, ...parameters);
        };
        const baseReadFile = resourceHost.readFile;
        resourceHost.readFile = function (fileName) {
            if (fileName.includes('virtual-analog:')) {
                const filePath = fileName.split('virtual-analog:')[1];
                const fileContent = baseReadFile.call(resourceHost, filePath) ||
                    'No Analog Markdown Content Found';
                // eslint-disable-next-line prefer-const
                const templateContent = constants_1.TEMPLATE_TAG_REGEX.exec(fileContent)?.pop()?.trim() || '';
                return templateContent;
            }
            return baseReadFile.call(resourceHost, fileName);
        };
        const fileExists = resourceHost.fileExists;
        resourceHost.fileExists = function (fileName) {
            if (fileName.includes('virtual-analog:') &&
                !fileName.endsWith('analog.d')) {
                return true;
            }
            return fileExists.call(resourceHost, fileName);
        };
    }
    resourceHost.readResource = async function (fileName) {
        const filePath = (0, paths_1.normalizePath)(fileName);
        const content = this.readFile(filePath);
        if (content === undefined) {
            throw new Error('Unable to locate component resource: ' + fileName);
        }
        if (fileName.includes('virtual-analog:')) {
            // read template sections, parse markdown
            const markedSetupService = new marked_setup_service_1.MarkedSetupService();
            const mdContent = markedSetupService
                .getMarkedInstance()
                .parse(content);
            return mdContent;
        }
        return content;
    };
    resourceHost.transformResource = async function (data, context) {
        // Only style resources are supported currently
        if (context.type !== 'style') {
            return null;
        }
        if (options.inlineStylesExtension) {
            // Resource file only exists for external stylesheets
            const filename = context.resourceFile ??
                `${context.containingFile.replace(/(\.analog)?\.ts$/, (...args) => {
                    // NOTE: if the original file name contains `.analog`, we turn that into `-analog.css`
                    if (args.includes('.analog')) {
                        return `-analog.${options?.inlineStylesExtension}`;
                    }
                    return `.${options?.inlineStylesExtension}`;
                })}`;
            let stylesheetResult;
            try {
                stylesheetResult = await transform(data, `${filename}?direct`);
            }
            catch (e) {
                console.error(`${e}`);
            }
            return { content: stylesheetResult?.code || '' };
        }
        return null;
    };
}
exports.augmentHostWithResources = augmentHostWithResources;
//# sourceMappingURL=host.js.map